@@ -2,36 +2,37 @@ import abc
 import logging
 import threading
 from queue import Empty, Full
-from typing import Callable, Union, List, Any, Optional, Tuple, NamedTuple
+from typing import Any, Callable, Generic, List, NamedTuple, Optional, Tuple, TypeVar, Union
 
 logger = logging.getLogger(__name__)
 
 Empty = Empty
 Full = Full
 
+QItem = TypeVar("QItem")
 
-class Queue(abc.ABC):
 
+class Queue(abc.ABC, Generic[QItem]):
     @property
-    def name(self):
+    def name(self) -> str:
         raise NotImplementedError
 
-    def get(self, block=True, timeout=None):
+    def get(self, block: bool = True, timeout: float = None) -> QItem:
         raise NotImplementedError
 
-    def put(self, item, block=True, timeout=None):
+    def put(self, item: QItem, block: bool = True, timeout: float = None):
         raise NotImplementedError
 
-    def qsize(self):
+    def qsize(self) -> int:
         raise NotImplementedError
 
-    def empty(self):
+    def empty(self) -> bool:
         return self.qsize() == 0
 
-    def put_nowait(self, item):
+    def put_nowait(self, item: QItem):
         return self.put(item, block=False)
 
-    def get_nowait(self):
+    def get_nowait(self) -> QItem:
         return self.get(block=False)
 
     def close(self):
@@ -42,7 +43,6 @@ class Queue(abc.ABC):
 
 
 class Topic(abc.ABC):
-
     @property
     def name(self) -> str:
         raise NotImplementedError
@@ -73,7 +73,6 @@ class RpcResponse(NamedTuple):
 
 
 class StubMethod:
-
     def __call__(self, *args, **kwargs):
         raise NotImplementedError
 
@@ -82,14 +81,13 @@ class StubMethod:
 
 
 class EventBus(abc.ABC):
-
     def run(self):
         raise NotImplementedError
 
     def close(self):
         raise NotImplementedError
 
-    def publish(self, event, channel=None):
+    def publish(self, event, channel=None) -> Optional[int]:
         raise NotImplementedError
 
     def subscribe(self, callback: Callable, channel=None, pattern=False):
@@ -101,7 +99,7 @@ class EventBus(abc.ABC):
     def queue(self, name: str) -> Queue:
         raise NotImplementedError
 
-    def topic(self, name: str, pattern: bool = False):
+    def topic(self, name: str, pattern: bool = False) -> Topic:
         raise NotImplementedError
 
     def stub(self, fn, timeout=None, multi=False) -> StubMethod:
@@ -141,7 +139,7 @@ class _WrapperTopic(Topic):
 
     def publish(self, event) -> int:
         if self.is_pattern:
-            raise ValueError('Cannot publish to pattern topic')
+            raise ValueError("Cannot publish to pattern topic")
         else:
             return publish(event, self.name)
 
@@ -166,7 +164,7 @@ def unsubscribe(callback, channel=None, pattern=False):
 
 
 def subscriber(*args, **kwargs):
-    if callable(args[0]):
+    if args and callable(args[0]):
         subscribe(args[0], *args[1:], **kwargs)
         return args[0]
 
@@ -199,7 +197,7 @@ def init(factory, start_bus=True):
 
 def publish(event, channel=None):
     if _bus is None:
-        logger.error('Event bus was not initialized, cannot publish message. Please run pymq.init')
+        logger.error("Event bus was not initialized, cannot publish message. Please run pymq.init")
         return
 
     return _bus.publish(event, channel)
@@ -207,8 +205,8 @@ def publish(event, channel=None):
 
 def queue(name) -> Queue:
     if _bus is None:
-        logger.error('Event bus was not initialized, cannot get queue. Please run pymq.init')
-        raise ValueError('Bus not set yet')
+        logger.error("Event bus was not initialized, cannot get queue. Please run pymq.init")
+        raise ValueError("Bus not set yet")
 
     return _bus.queue(name)
 
@@ -222,8 +220,8 @@ def topic(name, pattern=False):
 
 def stub(fn, timeout=None, multi=False) -> StubMethod:
     if _bus is None:
-        logger.error('Event bus was not initialized, cannot get stub. Please run pymq.init')
-        raise ValueError('Bus not set yet')
+        logger.error("Event bus was not initialized, cannot get stub. Please run pymq.init")
+        raise ValueError("Bus not set yet")
 
     return _bus.stub(fn, timeout, multi)
 
@@ -239,8 +237,8 @@ def expose(fn, channel=None):
 def unexpose(fn):
     if _bus is None:
         # FIXME: will not remote uninitialized skeletons
-        logger.error('Event bus was not initialized, cannot unexpose method. Please run pymq.init')
-        raise ValueError('Bus not set yet')
+        logger.error("Event bus was not initialized, cannot unexpose method. Please run pymq.init")
+        raise ValueError("Bus not set yet")
 
     return _bus.unexpose(fn)
 
@@ -248,12 +246,12 @@ def unexpose(fn):
 def start():
     with _lock:
         if _bus is None:
-            raise ValueError('Bus not set yet')
+            raise ValueError("Bus not set yet")
 
-        logger.debug('starting global event bus')
+        logger.debug("starting global event bus")
         global _runner
         if _runner is None:
-            _runner = threading.Thread(target=_bus.run, name='eventbus-runner')
+            _runner = threading.Thread(target=_bus.run, name="eventbus-runner")
             _runner.daemon = True
             _runner.start()
 
@@ -263,11 +261,11 @@ def shutdown():
     with _lock:
         if _runner is None:
             return
-        logger.debug('stopping global event bus')
+        logger.debug("stopping global event bus")
         if _bus is not None:
             _bus.close()
         _runner.join()
-        logger.debug('global event bus stopped')
+        logger.debug("global event bus stopped")
         _runner = None
         _bus = None
         _uninitialized_subscribers.clear()
